/********************************************************************
 BeClock KWin - effect plugin

 Copyright (C) 2009,2010 Thomas Lübking <thomas.luebking@web.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/


#ifndef BECLOCK_H
#define BECLOCK_H

#include <kwineffects.h>

#include <QObject>

namespace KWin
{
class GLTexture;

class BeClock :
#if ! KDE_IS_VERSION(4,6,30)
public QObject,
#endif
public Effect {
    Q_OBJECT
public:
    BeClock();
    ~BeClock();
    bool borderActivated( ElectricBorder border );
    void countdown(uint secs);
    void getAttention(int n);
    void reconfigure( ReconfigureFlags );
    void prePaintScreen( ScreenPrePaintData& data, int time );
    void paintScreen( int mask, QRegion region, ScreenPaintData& data );
    void postPaintScreen();
public slots: // -> 4.7 -> private
    void mouseChanged(const QPoint &pos, const QPoint &oldpos,
                      Qt::MouseButtons buttons, Qt::MouseButtons oldbuttons,
                      Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers);
protected:
    void timerEvent( QTimerEvent * );
private:
    QRect cornerRect( ElectricBorder border );
    void drawClock( QPainter &p, const QRect &r );
    QImage *clockImage();
    QImage *binaryImage( const QDateTime &dt );
    QImage *mayaImage( const QDateTime &dt );
    QImage *ringImage( const QDateTime &dt );
    QImage *digitImage( const QDateTime &dt, QRect r );
    QImage *halo( QImage *img, int r );
    void hide(bool smooth = true);
    bool mouseIsInAciveCorner( const QPoint &pos );
    void pollTheMouse(bool b);
    int screen() const;
    void show(bool smooth = true);
    void updateBuffer();

private slots:
    void attend();
    void init();
    void plainHide() { hide(false); }
    void toggle();
    void updateGeometry();

private:
    enum Type { Rings, Binary, Maya };
    bool imActive, iUse24h, iHideOnHover, iPollTheMouse, iFadeIn, iChime;
    bool iShowDigits, iShowRings, iShowTheDate, iShowTheDay, iShowTheSeconds;
    int myClock, myAlign, myScreen, myFadeTimer, myRemainingAttention, myPaintCounter;
    float myAlpha;
    ElectricBorder myCurrentCorner;
    QList<ElectricBorder> myElectricBorders;
    int myLastUpdateTime, myRemainingAnimationTime, myHomeTimeDiff;
    uint myCountDown, myCountDownAttention, myAttentionTimeout[2];
    QColor myBackground, myForeground;
    QRect myGeometry, myTargetGeometry, mySecondGeometry;
    QFont myFont;
    struct {
        int x,y;
    } myOffset;
    QPixmap *myBufferXR[2];
    GLTexture *myBufferGL[2];
    QString myDatePattern;
    Type myType;
};

}

#endif
